"""Constants used for LG webOS Smart TV."""
DOMAIN = "webostv"

DEFAULT_NAME = "LG webOS Smart TV"

ATTR_BUTTON = "button"
ATTR_COMMAND = "command"
ATTR_SOUND_OUTPUT = "sound_output"

CONF_ON_ACTION = "turn_on_action"
CONF_SOURCES = "sources"

SERVICE_BUTTON = "button"
SERVICE_COMMAND = "command"
SERVICE_SELECT_SOUND_OUTPUT = "select_sound_output"

LIVE_TV_APP_ID = "com.webos.app.livetv"

WEBOSTV_CONFIG_FILE = "webostv.conf"
